/// ## mov(register: u8, operand: u32)
/// - 0: opcode
/// - 1: register
/// - 2..5: operand
pub const MOV_INT: u8 = 0x01;

pub const MOV_LONG: u8 = 0x02;
pub const MOV_DOUBLE: u8 = 0x03;

/// ## mov(target_register: u8, source_register: u8)
/// 0: opcode
/// 1: target_register
/// 2: source_register
pub const MOV_REGISTER: u8 = 0x04;

/// ## alloc(register: u8, size: u32)
/// 0: opcode
/// 1: register
/// 2..5: size
pub const ALLOC: u8 = 0x05;

/// ## dealloc(register: u8)
/// 0: opcode
/// 1: register
pub const DEALLOC: u8 = 0x06;

/// ## mov_int_to(register: u8, offset: u32, operand: u32)
/// 0: opcode
/// 1: register
/// 2..5: offset
/// 6..9: operand
pub const MOV_INT_TO: u8 = 0x07;

pub const MOV_LONG_TO: u8 = 0x08;
pub const MOV_DOUBLE_TO: u8 = 0x09;
pub const MOV_REGISTER_TO: u8 = 0x0a;

#[derive(PartialEq, Eq, Clone, Debug)]
pub enum RegisterType {
    Int,
    Long,
    Double,
    Pointer,
}

pub fn add_mov_int(code: &mut Vec<u8>, register: u8, operand: u32) {
    code.push(MOV_INT);
    code.push(register);
    for b in operand.to_le_bytes() {
        code.push(b);
    }
}

pub fn add_mov_register(code: &mut Vec<u8>, target_register: u8, source_register: u8) {
    code.push(MOV_REGISTER);
    code.push(target_register);
    code.push(source_register);
}

pub fn add_alloc(code: &mut Vec<u8>, register: u8, size: u32) {
    code.push(ALLOC);
    code.push(register);
    for b in size.to_le_bytes() {
        code.push(b);
    }
}

pub fn add_dealloc(code: &mut Vec<u8>, register: u8) {
    code.push(DEALLOC);
    code.push(register);
}

pub fn add_mov_int_to(code: &mut Vec<u8>, register: u8, offset: u32, operand: u32) {
    code.push(MOV_INT_TO);
    code.push(register);
    for b in offset.to_le_bytes() {
        code.push(b);
    }
    for b in operand.to_le_bytes() {
        code.push(b);
    }
}
